@@ -1,5 +1,5 @@
 // Copyright Contributors to the OpenVDB Project
-// SPDX-License-Identifier: MPL-2.0
+// SPDX-License-Identifier: Apache-2.0
 
 #include <openvdb/openvdb.h>
 #include <openvdb/Types.h>
@@ -574,3 +574,42 @@ TEST_F(TestLeafMask, testTransientData)
     LeafType leaf3 = leaf;
     EXPECT_EQ(openvdb::Index32(5), leaf3.transientData());
 }
+
+TEST_F(TestLeafMask, testUnsafe)
+{
+    using namespace openvdb;
+    using LeafT = tree::LeafNode<ValueMask, 3>;
+    const Coord origin(-9, -2, -8);
+    LeafT leaf(origin, false, /*dummy=*/false);
+
+    EXPECT_FALSE(leaf.isValueOn(1));
+    EXPECT_TRUE(leaf.isValueOff(1));
+    EXPECT_EQ(leaf.getValueUnsafe(1), false);
+    bool value = true;
+    EXPECT_FALSE(leaf.getValueUnsafe(1, value));
+    EXPECT_EQ(value, false);
+
+    EXPECT_TRUE(leaf.isValueOff(32));
+    leaf.setValueOnUnsafe(32);
+    EXPECT_TRUE(leaf.isValueOn(32));
+    leaf.setValueOffUnsafe(32);
+    EXPECT_TRUE(leaf.isValueOff(32));
+    leaf.setActiveStateUnsafe(32, true);
+    EXPECT_TRUE(leaf.isValueOn(32));
+    leaf.setActiveStateUnsafe(32, false);
+    EXPECT_TRUE(leaf.isValueOff(32));
+
+    leaf.setValueOnlyUnsafe(32, true);
+    EXPECT_EQ(leaf.getValueUnsafe(32), true);
+    EXPECT_TRUE(leaf.isValueOn(32));
+    leaf.setValueOffUnsafe(32);
+    EXPECT_TRUE(leaf.isValueOff(32));
+
+    leaf.setValueOnUnsafe(33, true);
+    EXPECT_TRUE(leaf.isValueOn(33));
+    EXPECT_EQ(leaf.getValueUnsafe(33), true);
+    leaf.setValueOffUnsafe(33, false);
+    EXPECT_TRUE(leaf.isValueOff(33));
+    EXPECT_EQ(leaf.getValueUnsafe(33), false);
+}
+
